	list p=16f84
;**************************************************************
;*  	Pinbelegung
;*	----------------------------------	
;*	PORTA: 	0 - 
;*		1 -
;*		2 -	
;*		3 -
;*		4 Takteingang 64:1
;*	PORTB:	0 LCD Display E
;*		1 		
;*		2 LCD Display RS
;*		3 LCD Display R/W	
;*		4-7 LCD Display D4 .. D7 
;*	
;**************************************************************
;
;sprut (zero) Bredendiek 11/2000 (11/2001) (06/2002)
;
; 10-stelliger Frequenzzhler mit LCD-Display
; externer Vorteiler64:1 (U664B)
;
; Prozessor-Takt 10 MHz
;
; Mebereiche bezogen auf RA4:
;  4 Hz  ...  4 MHz	Auflsung  4 Hz
;  4 MHz ...  8 MHz	Auflsung  8 Hz
;  8 MHz ... 16 MHz	Auflsung 16 Hz
; 16 MHz ... 32 MHz	Auflsung 32 Hz
; 32 MHz ... 64 MHz	Auflsung 64 Hz
;
; Mebereiche bezogen auf U664B:
;  1 MHz ... 256 MHz	Auflsung 256 Hz
;256 MHz ... 512 MHz	Auflsung 512 Hz
;512 MHz ...   1 GHz	Auflsung 1 kHz
;  1 GHz ...   2 GHz	Auflsung 3 kHz
;  2 GHz ...   4 GHz	Auflsung 4 kHz
;
;**********************************************************
; Includedatei fr den 16F84 einbinden

	#include <P16f84.INC>


	ERRORLEVEL      -302    	;SUPPRESS BANK SELECTION MESSAGES


; Configuration festlegen: 16F84 628
; Power on Timer, kein Watchdog, HS-Oscillator, kein Brown out, kein LV-programming
;	__CONFIG	_PWRTE_ON & _WDT_OFF & _HS_OSC & _BODEN_OFF & _LVP_OFF	;PIC16F628
	__CONFIG	_PWRTE_ON & _WDT_OFF & _HS_OSC 				;PIC16F84


; Variablen festlegen

;w_copy		Equ	0x20	; nur fr INT
;s_copy		Equ	0x21	; nur fr INT
LcdDaten  	Equ	0x22
LcdStatus 	Equ	0x23
loops		EQU	0x24	; Wait-Schleife
loops2		EQU	0x25	; Wait-Schleife
BcdDaten	EQU	0x26
BcdDaten1	EQU	0x27
Fehler		EQU	0x28

; Dezimalstellen
HdG		EQU	0x29	;10 Milliarden
HdHM		EQU	0x2A	;9 HundertMillionen
HdZM		EQU	0x2B	;8 ZehnMillionen
HdM		EQU	0x2C	;7 Millionen
HdHT		EQU	0x2D	;6 Hunderttausender
HdZT		EQU	0x2E	;5 Zehntausender
HdT		EQU	0x2F	;4 Tausender
HdH		EQU	0x30	;3 Hunderter
HdZ		EQU	0x31	;2 Zehner
HdE		EQU	0x32	;1 Einer
HdX		EQU	0x33	; Puffer fr eine Dezimalstelle

; 40-Bit Rechenregister
xw0		EQU	0x34	;LSB
xw1		EQU	0x35	; I 
xw2		EQU	0x36	; I 
xw3		EQU	0x37	;MSB
xw4		EQU	0x38	;MSB

; 40-Bit Rechenregister
f0		EQU	0x39	;LSB
f1		EQU	0x3A	; I 
f2		EQU	0x3B	; I 
f3		EQU	0x3C	;MSB
f4		EQU	0x3D	;MSB

; 40 Bit Frequenzzhler
BIN4		EQU	0x3E	;LSB
BIN41		EQU	0x3F	; I 
BIN42		EQU	0x40	; I 
BIN43		EQU	0x41	;MSB
BIN44		EQU	0x42	;MSB

; Hilfszellen fr Frequenzmeschleife
HLTest		EQU	0x43	; HL-Flanken-Detektorn fr Timer0-berlauf
HighCOU		EQU	0x44	; high-Teil des Schleifenzhlers
LowCOU		EQU	0x45	; low-Teil des Schleifenzhlers
PreSC		EQU	0x46	; Vorteiler fr die richtige Messung
; bis 0x46 belegt
; beim 16F84 ist max. 0x4F

; Konstanten festlegen

PORTC	equ	PORTB		; LCD-Control-Port
PORTD	equ	PORTB		; LCD-Daten-Port
LcdE	equ	0		; enable Lcd
LcdRw	equ	3		; read Lcd
LcdRs	equ	2		; Daten Lcd (nicht control)	
Ini_con Equ	B'00000000'	; TMR0 -> Interupt disable
Ini_opt	Equ	B'00000010'	; pull-up

;********************************************************
; Das Programm beginnt mit der Initialisierung

Init	
	bsf     STATUS, RP0	; Bank 1
	movlw   Ini_opt     	; pull-up on
	movwf   OPTION_REG 
	movlw	B'11111000'	; RA0 .. RA2 outputs, RA3, RA4 input 
	movwf	TRISA		; 
	movlw	B'00000000'	; PortB alle outputs 
	movwf	TRISB
	bcf     STATUS, RP0	; Bank 0
	clrf	PORTA		
	clrf	PORTB		
				
	movlw   Ini_con     	; Interupt disable
	movwf   INTCON   

	call	InitLCD		; Display initialisieren

; am LCD "Hallo" ausgeben
	
	movlw	'H'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'a'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'l'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'l'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'o'
	movwf	LcdDaten
	call	OutLcdDaten

; timer1 vorbereiten
	bsf     STATUS, RP0		; Bank 1
	bsf	OPTION_REG, T0CS	; RA4-Takt zum Timer0
	bsf	OPTION_REG, PSA		; noch ohne Vorteiler
	bcf     STATUS, RP0		; Bank 0

	movlw	1
	movwf	PreSC			; 4:1

;*****************************************************************
; (HighCOU,LowCOU)x20Takte lang wird gezhlt
; das 24-Bit Ergebnis steht in BIN4+2 (MSB) bis BIN4 (LSB)
; virtuell steht davor BIN+3=0
; das mach dann 32Bit
; 
; fr Autorange: 1MHz Resolution bei PT=256:1
; 256 us x 1 MHz / 256 = 1
; 256 us = 640 Takte = 32 x 20 Takte , 00,20h=32d
;
;
; fr die Messung 500 ms
; 10MHz ; 500 ms = 1250000 Takte = 62500 x 20 Takte, 0,f4,24h=62500d   -Default
; 10MHz ; 250 ms =  625000 Takte = 32250 x 20 Takte, 0,7A,12h=32250d
; 10MHz ; 125 ms =  312500 Takte = 15625 x 20 Takte, 0,3D,09h=15625d
;
Frequ_Loop
;Autorange
	clrf	BIN4+1 		; clear next counter byte
	clrf	BIN4+2 		; clear next counter byte
	clrf	BIN4+3 		; clear next counter byte
	clrf	BIN4+4 		; clear next counter byte
	clrf	HLTest	 	; initialize TMR0 overflow detector
	movlw	00h+1 		; high loop counter for 256us
	movwf	HighCOU 	; HighCOU = hi byte counter
	movlw	20h+1 		; 00020h=.32 cycles=640,T=256us
	movwf	LowCOU 		; LowCOU = lo byte counter
	movlw	0x07	 	; PRESC = prescaler factor selected
	call	Messung

; Einstellung des Vorteilers anhand des Meergebnisses
; BIN4 enthlt Eingangsfrequenz in MHz
; >32 MHz 32:1 - 111x xxxx - PS=4
; >16 MHz 16:1 - 1111 xxxx - PS=3
; > 8 MHz  8:1 - 1111 1xxx - PS=2
; > 4 MHz  4:1 - 1111 11xx - PS=1
; > 2 MHz  2:1 - 1111 111x - PS=0
; < 2 MHz  2:1 -             PS=0

;	movfw	BIN4		; DEBUG: Meergebnis in MHz anzeigen
;	Call	Bcd4Bit

; falls die RA4-Eingangsfrequenz unter 2MHz liegt, wird trotzdem 2MHz
; eingestellt, da ein kleinerer PS-Wert als 0 nicht geht
; den PS herauszunehmen wre zu umstndlich
	bsf	BIN4,1		; failsave falls freq <2 MHz
	movlw	0x07
	movwf	PreSC
PSDec
	decf	PreSC, f
	rlf	BIN4, f
	btfss	STATUS, C
	goto	PSDec

;	movfw	PreSC		; DEBUG: PS anzeigen
;	Call	Bcd4Bit	

	nop
	nop

;Messung mit optimaler Auflsung
	clrf	BIN4+1 		; clear counter 
	clrf	BIN4+2 		; clear counter 
	clrf	BIN4+3 		; clear counter 
	clrf	BIN4+4 		; clear counter 
	clrf	HLTest	 	; initialize TMR0 overflow detector
	movlw	0f4h+1 		; high loop counter for 500 ms
	movwf	HighCOU 	; HighCOU = hi byte counter
	movlw	24h+1 		; 0f424h=.62500 cycles=1250000,T=500 ms
	movwf	LowCOU 		; LowCOU = lo byte counter
	movf	PreSC,W 	; PRESC = Vorteiler-Faktor
	call	Messung
;Ende der Zhlroutine
; 500 ms
; 2:1 Fmax =  5 MHz = 2,5 Mill-Schwingungen = 1,25 Mill = 13 12 D0 h
; 4:1 Fmax = 10 MHz =   5 Mill-Schwingungen = 1,25 Mill = 13 12 D0 h


;************************************************************************
; Zhlwert in Frequenz umrechnen
; Mezeit kompensieren
	call	Mal2		; 500ms -> 1s

; externen 64:1-Vorteiler kompensieren
	call	Mal2		;   2 : 1
	call	Mal2		;   4 : 1
	call	Mal2		;   8 : 1
	call	Mal2		;  16 : 1
	call	Mal2		;  32 : 1
	call	Mal2		;  64 : 1

	call	Mal2		; 128 : 1
	call	Mal2		; 256 : 1	
;	call	Mal2		; 512 : 1

; internen Timer0-Vorteiler kompensieren
	movfw	PreSC
	movwf	Fehler		; sichern fr Berechnung der Frequenz
	incf	Fehler, f
mal
	call	Mal2		; PS kompensieren
	decfsz	Fehler,f
	goto	mal

;Kursor zurck
	movlw	B'00000011'	; 8 cursor home, cursor home
	call	OutLcdControl

; 40 Bit zur Anzeige bringen
	movfw	BIN4
	movwf	f0
	movfw	BIN4+1
	movwf	f1
	movfw	BIN4+2
	movwf	f2
	movfw	BIN4+3
	movwf	f3
	movfw	BIN4+4
	movwf	f4
;Binr zu ASCII Wandlung mit Ausgabe
	call	OutDez40	; 'x xxx xxx xxx'
	call	OutHz		; ' Hz'

;	movfw	PreSC		; DEBUG	 aktuell verwendeten PS anzeigen
;	Call	Bcd4Bit

	goto	Frequ_Loop	; und alles noch mal von vorne


;***********************************************************************************
; die eigentliche  Meroutine
; Mezeit = 20Takte x (HighCOU,LowCOU)
; Prescaler in w
Messung
	addlw	20h 		; for PRESC 0,1,2,3... w=20h,21h,22h,23h...
				; TOSE=0, TOCS=1, PSA=0, PS=0..7
	bsf	STATUS,RP0 	; bank 1
	movwf	OPTION_REG 	; 
	bcf	STATUS,RP0 	; bank 0
	clrf	TMR0 		; initialize TMR0

Loop500A
	nop			; warten
	nop			; warten
Loop500
	movf	TMR0, W		; 1 1 1 only place where TMR0 is read
	movwf	BIN4		; 1 1 1 rtcc ---> freq0
	rlf	BIN4, W		; 1 1 1 carry <--- TMR0.7
	rlf	HLTest, F	; 1 1 1 rotor <--- carry
	movf	HLTest, W	; 1 1 1 HLTest=TMR0 overflow detect
	andlw	3		; 1 1 1 mask 2 LSbs for edge detect
	xorlw	2		; 1 1 1 00000000 if 1 <--- 0
	btfss	STATUS, Z	; 1 2 2 | skip if TMR0 overflow
	goto	NotOvf1		; 2 - - | if nz
				;       | 5T any case
	incf	BIN4+1, F	; - 1 1 | nsb
NotOvf1 btfsc	STATUS, Z	; 2 2 1 | skip MSB adv ifnot overflow
	incf	BIN4+2, F 	; - - 1 | msb
	nop			; warten
	nop			; warten
	nop			; warten
	decfsz	LowCOU, F 	; 1 (2) lo loop counter
	goto	Loop500A 	; 2 (-) 20T total
	decfsz	HighCOU, F 	; (1) hi loop counter
	goto	Loop500 	; (2) 20T total

	return


;*******************************************************************************
;Multiplikation: 40-Bit x 2 
Mal2
	bcf	STATUS,C
	rlf	BIN4, f
	rlf	BIN4+1, f
	rlf	BIN4+2, f
	rlf	BIN4+3, f
	rlf	BIN4+4, f
	return

;*******************************************************************************
; String ' Hz' am LCD ausgeben
OutHz
	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'H'
	movwf	LcdDaten
	call	OutLcdDaten
	movlw	'z'
	movwf	LcdDaten
	call	OutLcdDaten
	return

;*****************************************************************	
;Zeitverzgerung um loops * 1 ms
;10MHz -> 2,5MHz -> 2500T

WAIT
top      movlw   .249           ; timing adjustment variable (1ms)  10MHz 249*10=2490T
         movwf   loops2
top2     nop                     ; sit and wait (Schleifenlnge 10T)
         nop
         nop
         nop
         nop
         nop
	 nop			
         decfsz  loops2, F      ; inner loops complete?
         goto    top2           ; no, go again
                                ;
         decfsz  loops, F       ; outer loops complete?
         goto    top            ; no, go again
         retlw   0              ; yes, return from subWAIT

;**************************************************************
; LCD-Routinen
;	
; Initialisierung des LCD-Displays
InitLCD
	movlw	D'255'		; 250 ms Pause nach dem Einschalten
	movwf	loops	
	call	WAIT		

	movlw	B'00110000'	; 1
	movwf	PORTB
	bsf	PORTB, LcdE
	nop	
	bcf	PORTB, LcdE
	
	movlw	D'50'		; 50 ms Pause
	movwf	loops
	call	WAIT
	
	movlw	B'00110000'	; 2
	call	Control8Bit
	movlw	B'00110000'	; 3
	call 	Control8Bit
	movlw	B'00100000'	; 4
	call 	Control8Bit

	movlw	B'00000001'	; lschen und cusor home
	call	OutLcdControl	
	movlw	B'00101000'	; 5 function set, 4-bit  2-zeilig,  5x7
	call	OutLcdControl	
	movlw	B'00001000'	; 6 display off
	call	OutLcdControl
	movlw	B'00000110'	; 7 entry mode, increment, disable display-shift
	call	OutLcdControl
	movlw	B'00000011'	; 8 cursor home, cursor home
	call	OutLcdControl
	movlw	B'00001100'	; 9 display on, cursor off, cursor flash off
	call	OutLcdControl
	return

; ein Steuerbyte 8-bittig bertragen
Control8Bit
	movwf	PORTB
	bsf	PORTB, LcdE
	nop
	bcf	PORTB, LcdE
	movlw	D'10'
	movwf	loops
	call 	WAIT
	return

; darauf warten, da das Display bereit zur Datenannahme ist
LcdBusy
        bsf     STATUS, RP0	; make Port B4..7 input
	movlw	B'11110000'
	iorwf   TRISB, f 
        bcf     STATUS, RP0
BusyLoop		
	bcf	PORTC, LcdRs
	bsf	PORTC, LcdRw	; Lesen
	bsf	PORTC, LcdE
	nop
	movf	PORTD, w
	movwf	LcdStatus
	bcf	PORTC, LcdE
	nop
	bsf	PORTC, LcdE	; Enable
	nop
	bcf	PORTC, LcdE
	btfsc	LcdStatus, 7	; teste bit 7
	goto	BusyLoop
	bcf	PORTC, LcdRw
        bsf     STATUS, RP0	; make Port B4..7 output
	movlw	B'00001111'
	andwf   TRISB, f    
        bcf     STATUS, RP0
	return	

; ein Byte mit Steuerdaten von LcdDaten zum Display bertragen
OutLcdControl
	movwf	LcdDaten
	call	LcdBusy
	movf	LcdDaten, w
	andlw	H'F0'
	movwf	PORTD		; Hi-teil Daten schreiben
	bsf	PORTC, LcdE
	nop
	bcf	PORTC, LcdE	; Disable LcdBus
	swapf	LcdDaten, w
	andlw	H'F0'
	movwf	PORTD		; Lo-teil Daten schreiben
	bsf	PORTC, LcdE
	nop
	bcf	PORTC, LcdE	; Disable LcdBus
	return

; ein Datenbyte von LCDDaten zum Display bertragen
OutLcdDaten
	movwf	LcdDaten
	call	LcdBusy
	movf	LcdDaten, w
	andlw	H'F0'
	movwf	PORTD		; Hi-teil Daten schreiben
	bsf	PORTC, LcdRs	; Daten
	bsf	PORTC, LcdE	; Enable LcdBus
	nop
	bcf	PORTC, LcdE	; Disable LcdBus	
	swapf	LcdDaten, w
	andlw	H'F0'
	movwf	PORTD		; Lo-teil Daten schreiben
	bsf	PORTC, LcdRs	; Daten
	bsf	PORTC, LcdE
	nop
	bcf	PORTC, LcdE	; Disable LcdBus	
	bcf	PORTC, LcdRs	;
	return


;***********************************************************
; 40 Bit Wert (f4,f3,f2,f1,f0) 10-stellig auf LCD dezimal anzeigen
OutDez40			;40-bit (f4,f3,f2,f1,f0) als Dez zum Lcd
	call	Hex2Dez40	;40 Bit to BCD

	clrf	Fehler		; Leading0 vorbereiten


	movfw	HdG
	Call	Leading0	; 1 xxx xxx xxx 

	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten

	movfw	HdHM	
	Call	Leading0	; 1 1xx xxx xxx 
	movfw	HdZM
	Call	Leading0	; 1 11x xxx xxx 
	movfw	HdM
	Call	Leading0	; 1 111 xxx xxx 

	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten

	movfw	HdHT
	Call	Leading0	; 1 111 1xx xxx 
	movfw	HdZT
	Call	Leading0	; 1 111 11x xxx 

;8 Zeichen fertig 
;ACHTUNG: die folgenden 2 Zeilen nur bei einem 1-Zeiligen Display 1x16 einfgen !!
;	movlw	B'11000000'	; Sprung zur virtuellen 2. Zeile bei 1x16 
;	call	OutLcdControl	; Sprung zur virtuellen 2. Zeile bei 1x16 

	movfw	HdT
	Call	Leading0	; 1 111 111 xxx 

	movlw	' '
	movwf	LcdDaten
	call	OutLcdDaten

	movfw	HdH
	Call	Leading0	; 1 111 111 1xx 
	movfw	HdZ
	Call	Leading0	; 1 111 111 11x 
	movfw	HdE
	Call	Bcd4Bit		; 1 111 111 111 
	return

Leading0
	iorwf	Fehler, f
	movf	Fehler, f	; Test auf 0
	btfss	STATUS, Z	; bisher alles '0'
	goto	Bcd4Bit		; nein
	movlw	' '		; ja
	movwf	LcdDaten
	goto	OutLcdDaten

;**************************************************************
; 16 Bit Wert (f1,f0) auf LCD dezimal anzeigen
OutDez16			;16-bit (f0,f1) als Dez zum Lcd
	call	Hex2Dez16
	movfw	HdZT
	Call	Bcd4Bit
	movfw	HdT
	Call	Bcd4Bit

	movlw	' '
	movwf	LcdDaten

	movfw	HdH
	Call	Bcd4Bit
	movfw	HdZ
	Call	Bcd4Bit
	movfw	HdE
	Call	Bcd4Bit
	return

;1 000 000 000 = 3B9A CA00 h
;  100 000 000 = 05F5 E100 h
;   10 000 000 = 0098 9680 h
;    1 000 000 = 000F 4240 h
;      100 000 = 0001 86A0 h
;       10 000 = 0000 2710 h
;        1 000 = 0000 03E8 h
;          100 = 0000 0064 h
;           10 = 0000 000A h
;            1 = 0000 0001 h

Hex2Dez40			; 40-bit(f4,f3,f2,f1,f0) in 8-stellen Bcd (G,HM,ZM,M,HT,ZT,T,H,Z,E)

	clrf	xw4		; 1 000 000 000 = 00 3B 9A CA 00 h	
	movlw	0x3B		
	movwf	xw3
	movlw	0x9A		
	movwf	xw2
	movlw	0xCA
	movwf	xw1
	movlw	0x00
	movwf	xw0
	call	Hex2Dez1	; 1 000 000 000er
	movfw	HdX
	movwf	HdG

	clrf	xw4		; 100 000 000 = 00 05 F5 E1 00 h	
	movlw	0x05	
	movwf	xw3
	movlw	0xF5		
	movwf	xw2
	movlw	0xE1
	movwf	xw1
	movlw	0x00
	movwf	xw0
	call	Hex2Dez1	; 100 000 000er
	movfw	HdX
	movwf	HdHM

	clrf	xw4		; 10 000 000 = 00 00 98 96 80 h
	clrf	xw3		
	movlw	0x98		
	movwf	xw2
	movlw	0x96
	movwf	xw1
	movlw	0x80
	movwf	xw0
	call	Hex2Dez1	; 10 000 000er
	movfw	HdX
	movwf	HdZM

	clrf	xw4		; 1 000 000 = 00 00 0F 42 40 h
	clrf	xw3	
	movlw	0x0F		
	movwf	xw2
	movlw	0x42
	movwf	xw1
	movlw	0x40
	movwf	xw0
	call	Hex2Dez1	; 1 000 000er
	movfw	HdX
	movwf	HdM

	clrf	xw4		; 100 000 = 00 00 01 86 A0 h
	clrf	xw3
	movlw	0x01		
	movwf	xw2
	movlw	0x86
	movwf	xw1
	movlw	0xA0
	movwf	xw0
	call	Hex2Dez1	; 100 000er
	movfw	HdX
	movwf	HdHT

Hex2Dez16			; 16-bit(f1,f0) in 5-stellen Bcd (ZT,T,H,Z,E)				
	movlw	0x27		; 10 000 = 00 00 00 27 10 h
	movwf	xw1
	clrf	xw2
	clrf	xw3
	clrf	xw4
	movlw	0x10
	movwf	xw0
	call	Hex2Dez1	; 10 000er
	movfw	HdX
	movwf	HdZT

	movlw	0x03		; 1 000 = 00 00 00 03 E8 h
	movwf	xw1
	clrf	xw2
	clrf	xw3
	clrf	xw4
	movlw	0xE8
	movwf	xw0
	call	Hex2Dez1	; 1000er
	movfw	HdX
	movwf	HdT
Hex2Dez8			; 8-bit (f1, f0) in 3-stellen BCD
	movlw	0x00		; 100 =  00 00 00 00 64 h
	movwf	xw4
	movwf	xw3
	movwf	xw2
	movwf	xw1
	movlw	0x64
	movwf	xw0
	call	Hex2Dez1	; 100er
	movfw	HdX
	movwf	HdH

	movlw	0x00		; 10 = 00 00 00 00 0A h
	movwf	xw4
	movwf	xw3
	movwf	xw2
	movwf	xw1
	movlw	0x0A
	movwf	xw0
	call	Hex2Dez1	; 10er
	movfw	HdX
	movwf	HdZ

	movfw	f0
	movwf	HdE
	return

Hex2Dez1
	clrf	HdX
	decf	HdX, f
HdLoop
	incf	HdX, f
	call	Sub40		;
	btfss	STATUS, C	;berlauf
	goto	HdLoop		;Stelle 1 mehr
	call	Add40
	return

; 32 Bit Subtraktion, bei berlauf (neg. Ergebnis) ist C gesetzt
Sub40				; 40 bit f:=f-xw   calc=xw cnt=f
	clrf	Fehler		; extraflags lschen    
	movf    xw0, w		; f0=f0-xw0
	subwf   f0, f

	btfsc   STATUS,C
	goto    sb
	movlw   0x01		; borgen von f1
	subwf   f1, f

	btfsc   STATUS,C
	goto    sb
	subwf   f2    ,f        ; borgen von f2

	btfsc   STATUS,C
	goto    sb
	subwf   f3    ,f        ; borgen von f3

	btfsc   STATUS,C
	goto    sb
	subwf   f4    ,f        ; borgen von f4

	btfss   STATUS,C
	bsf	Fehler, C	; unterlauf

sb			
	movf    xw1, w		; f1=f1-xw1
	subwf   f1, f

	btfsc   STATUS,C
	goto    sb0
	movlw   0x01		; borgen von f2
	subwf   f2, f

	btfsc   STATUS,C
	goto    sb0
	subwf   f3    ,f        ; borgen von f3

	btfsc   STATUS,C
	goto    sb0
	subwf   f4    ,f        ; borgen von f4
	btfss   STATUS,C
	bsf	Fehler, C	; unterlauf

sb0     movf    xw2, w		; f2=f2-xw2
	subwf   f2, f

	btfsc   STATUS,C
	goto    sb1
	movlw   0x01		; borgen von f3
	subwf   f3    ,f

	btfsc   STATUS,C
	goto    sb1
	subwf   f4, f		; borgen von f4

        btfss   STATUS,C
	bsf	Fehler, C	; Unterlauf

sb1     movf    xw3,w		; f3=f3-xw3
        subwf   f3, f

        btfsc   STATUS,C
        goto    sb2
        movlw   0x01		
        subwf   f4, f		; borgen von f4

        btfss   STATUS,C
       	bsf	Fehler, C	; Unterlauf

sb2     movf    xw4,w		; f4=f4-xw4
        subwf   f4    ,f

        btfss   STATUS,C
 	bsf	Fehler, C	; Unterlauf

	bcf	STATUS, C
	btfsc	Fehler, C
	bsf	STATUS, C
	return



;40 bit Adition, C-Flag bei berlauf gesetzt 
;********************************************************************* 
;*  
;* addiere xw (40-byte value) mit f (40-byte value). 
;* Input  variablen: f (40-bit value), xw (40-bit value) 
;* Output variablen: f (40-bit value), 41. bit in STATUS,C 
;********************************************************************* 
Add40 				; 40-bit add: f = f + xw
	movf	xw0,W		; low byte
	addwf	f0,F 		; low byte add

	movf	xw1,W 		; next byte
	btfsc	STATUS,C 	; berspringe falls C nicht gesetzt
	incfsz	xw1,W 		; addiere C falls gesetzt 
	addwf	f1,F 		; next byte add wenn NZ
	
	movf	xw2,W 		; next byte
	btfsc	STATUS,C 	; berspringe falls C nicht gesetzt
	incfsz	xw2,W 		; addiere C falls gesetzt 
	addwf	f2,F 		; next byte add wenn NZ
	
	movf	xw3,W 		; high byte
	btfsc	STATUS,C 	; berspringe falls C nicht gesetzt
	incfsz	xw3,W 		; addiere C falls gesetzt 
	addwf	f3,F 		; high byte add wenn NZ

	movf	xw4,W 		; high byte
	btfsc	STATUS,C 	; berspringe falls C nicht gesetzt
	incfsz	xw4,W 		; addiere C falls gesetzt 
	addwf	f4,F 		; high byte add wenn NZ
	
	return 			; fertig




OutBcd				;8-Bit als BCD ausgeben
	movwf	BcdDaten	;8-Bit kopieren
	movwf	BcdDaten1
	rrf	BcdDaten, f
	rrf	BcdDaten, f
	rrf	BcdDaten, f
	rrf	BcdDaten, f	;4 High-Bit in BcdDaten
	movlw	B'00001111'
	andwf	BcdDaten, f	; / 
	andwf	BcdDaten1, f	; 4-Low-Bit in BcdDaten1
	movfw	BcdDaten
	call	Bcd4Bit		;4 High-Bit ausgeben
	movfw	BcdDaten1
	call	Bcd4Bit		;4 Low-Bit ausgeben
	return
Bcd4Bit				;low-4 Bit als BCD ausgeben
	movwf	BcdDaten
	movlw	B'00110000'
	ADDwf	BcdDaten, f	;ASCII-wandeln (+48)
	movlw	B'00111010'
	subwf	BcdDaten, w
	btfss	STATUS, C	;Test auf A ... F
	goto	BcdOk
	movlw	.7
	addwf	BcdDaten, f	;korrigiere A...F (+7)
BcdOk
	movfw	BcdDaten
	call	OutLcdDaten
	return

	end		
